//
//  SettingsViewController.m
//  iTimesheet
//
//  Created by Mark Brindle on 25/03/2011.
//  Copyright 2011 ARKEMM Software Limited. All rights reserved.
//

// Here we demonstratre checking connection details (wifi, cell etc) and also
// downloading data.  Obviously getting the top 10 songs from Apple's iTunes
// store is not really useful for our timesheet app - but it does indicate how
// details can be obtained and manipulated.

#import "SettingsViewController.h"

#import <SystemConfiguration/SystemConfiguration.h>
#import <netdb.h>
#import "UIDevice-Reachability.h"

@implementation SettingsViewController
@synthesize connectionText;
@synthesize downloadTable;

- (id)init {
  self = [super initWithNibName:nil bundle:nil];
  // Initialise our downloads array
  downloads = nil;
  return self;
}

- (id)initWithNibName:(NSString *)nibNameOrNil 
               bundle:(NSBundle *)nibBundleOrNil {
	// We're not interested in the parameters passed to us here; just call init
  // on ourself.
	return [self init];
}

- (void)dealloc
{
  [downloads release];
  [connectionText release];
  [downloadTable release];
  [super dealloc];
}

- (void)didReceiveMemoryWarning
{
  // Releases the view if it doesn't have a superview.
  [super didReceiveMemoryWarning];
  
  // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
  [super viewDidLoad];
  // Do any additional setup after loading the view from its nib.
  
  // Register as a Reachability watcher
  [UIDevice scheduleReachabilityWatcher:self];
  
  
  //  BOOL connected;
  //  connected = [self connectedToNetwork];
  //  
  //  // Test for a network connection
  //  //  [self checkConnection];
  //  
  //  [[self connectionText] setText: connected ?
  //    @"Connected to network" : 
  //    @"Not connected to network"];
  //  
  //  if (connected) {
  //    // Do networky stuff
  //  }
}

- (void)viewDidUnload
{
  [self setConnectionText:nil];
  [self setDownloadTable:nil];
  [super viewDidUnload];
  // Release any retained subviews of the main view.
  // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  // Return YES for supported orientations
  return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void)viewWillAppear:(BOOL)animated {
  [super viewWillAppear:animated];
  // Check connection to network when view is about to be displayed.
  [self checkConnection];
  if (downloads == nil) {
    downloads = [[NSMutableArray alloc] init];
  }
}

- (void)loadSongs {
  [downloads removeAllObjects];
	[[self downloadTable] reloadData];
  
	// Construct the web service URL
	NSURL *url = [NSURL URLWithString:@"http://ax.itunes.apple.com/"
                @"WebObjects/MZStoreServices.woa/ws/RSS/topsongs/"
                @"limit=10/xml"];
  
	// Create a request object with that URL								  
	NSURLRequest *request = [NSURLRequest requestWithURL:url 
                                           cachePolicy:NSURLRequestReloadIgnoringCacheData 
                                       timeoutInterval:30]; 
  
  // Clear out the existing connection if there is one 
  if (connectionInProgress) { 
    [connectionInProgress cancel]; 
    [connectionInProgress release]; 
  } 
	
	// Create and initiate the connection
  connectionInProgress = [[NSURLConnection alloc] initWithRequest:request 
                                                         delegate:self 
                                                 startImmediately:YES]; 
  
	// Instantiate the object to hold all incoming data
  [xmlData release]; 
  xmlData = [[NSMutableData alloc] init]; 
  
}

- (IBAction)doDownload:(id)sender {
	[self loadSongs];
  
}

#pragma mark - Examples taken from The Big Nerd Ranch Guide

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data 
{ 
  [xmlData appendData:data]; 
} 

- (void)connectionDidFinishLoading:(NSURLConnection *)connection 
{ 
  NSXMLParser *parser = [[NSXMLParser alloc] initWithData:xmlData]; 
  [parser setDelegate:self]; 
  [parser parse]; 
  [parser release]; 
	[[self downloadTable] reloadData];
} 

- (void)parser:(NSXMLParser *)parser 
didStartElement:(NSString *)elementName 
  namespaceURI:(NSString *)namespaceURI 
 qualifiedName:(NSString *)qName 
    attributes:(NSDictionary *)attributeDict 
{ 
	if([elementName isEqual:@"entry"])
    {
		waitingForEntryTitle = YES;
    }	
  if ([elementName isEqual:@"title"] && waitingForEntryTitle) { 
    titleString = [[NSMutableString alloc] init]; 
  } 
} 

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string { 
  [titleString appendString:string]; 
} 

- (void)parser:(NSXMLParser *)parser 
 didEndElement:(NSString *)elementName 
  namespaceURI:(NSString *)namespaceURI 
 qualifiedName:(NSString *)qName 
{ 
  if ([elementName isEqual:@"title"] && waitingForEntryTitle) { 
    [downloads addObject:titleString];
    [titleString release]; 
    titleString = nil; 
  } 
	if([elementName isEqual:@"entry"])
    {
		waitingForEntryTitle = NO;
    }
} 

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error 
{ 
  [connectionInProgress release]; 
  connectionInProgress = nil; 
  [xmlData release]; 
  xmlData = nil; 
  NSString *errorString = [NSString stringWithFormat:@"Fetch failed: %@", 
                           [error localizedDescription]]; 
  UIActionSheet *actionSheet = [[UIActionSheet alloc] initWithTitle:errorString 
                                                           delegate:nil 
                                                  cancelButtonTitle:@"OK" 
                                             destructiveButtonTitle:nil 
                                                  otherButtonTitles:nil]; 
  [actionSheet showInView:[[self view] window]];
  [actionSheet autorelease]; 
}    

#pragma mark - Network methods

- (BOOL) connectedToNetwork
{
  // Create zero addy
  struct sockaddr_in zeroAddress;
  bzero(&zeroAddress, sizeof(zeroAddress));
  zeroAddress.sin_len = sizeof(zeroAddress);
  zeroAddress.sin_family = AF_INET;
  
  // Recover reachability flags
  SCNetworkReachabilityRef defaultRouteReachability = SCNetworkReachabilityCreateWithAddress(NULL, (struct sockaddr *)&zeroAddress);    
  SCNetworkReachabilityFlags flags;
  
  BOOL didRetrieveFlags = SCNetworkReachabilityGetFlags(defaultRouteReachability, &flags);
  CFRelease(defaultRouteReachability);
  
  if (!didRetrieveFlags) {
    NSLog(@"Error. Could not recover network reachability flags.");
    return NO;
  }
  
  BOOL isReachable = ((flags & kSCNetworkFlagsReachable) != 0);
  BOOL needsConnection = ((flags & kSCNetworkFlagsConnectionRequired) != 0);
  return (isReachable && !needsConnection) ? YES : NO;
}

- (void)checkConnection {
  NSString *results = [NSString stringWithFormat:@"Network %@available %@%@", 
                       ([UIDevice networkAvailable] ? @"" : @"un"),
                       ([UIDevice activeWLAN] ? @"[WiFi]" : @""),
                       ([UIDevice activeWWAN] ? @"[Cell]" : @"")];
	[[self connectionText] setText:results];
}

#pragma mark - Reachability Monitoring (via Erica Sadun-The iPhone Developer's Cookbook)

- (void) reachabilityChanged
{
	[self checkConnection];
}

#pragma mark Table view methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  // Return the number of sections.
  return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
  return [downloads count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath 
{
  UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"UITableViewCell"];
  if (cell == nil) {
    cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault 
                                   reuseIdentifier:@"UITableViewCell"] autorelease];
  }
  
	[[cell textLabel] setText:[downloads objectAtIndex:[indexPath row]]];
  
  return cell;
}

@end
